/*******************************************************************************
 * Copyright (c) 2011 Oracle America, Inc.  All rights reserved.
 * This source file created for use with the eSWT library is made available
 * under the terms of the Eclipse Public License v1.0.
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/

package org.eclipse.swt.widgets;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;


public class WmAdfImage
  extends Canvas
{
  private boolean isFocusable;
  private Image m_image;

  private PaintListener m_paintListener = new PaintListener()
  {
    public void paintControl(PaintEvent paintEvent)
    {
      if (m_image != null)
      {
        paintEvent.gc.drawImage(m_image, 0, 0);

        if (isFocusControl())
        {
          Point size = getSize();
          paintEvent.gc.drawFocus(0, 0, size.x, size.y);
        }
      }
    }
  };

  private FocusListener m_focusListener = new FocusListener()
  {
    public void focusGained(FocusEvent focusEvent)
    {
      redraw();
    }

    public void focusLost(FocusEvent focusEvent)
    {
      redraw();
    }
  };

  public WmAdfImage(Composite parent, int style)
  {
    super(parent, style);
    addPaintListener(m_paintListener);
    addFocusListener(m_focusListener);
  }

  boolean isTabGroup()
  {
    return false;
  }

  boolean isTabItem()
  {
    return isFocusable;
  }

  public boolean getFocusable()
  {
    return isFocusable;
  }

  public void setFocusable(boolean focusable)
  {
    isFocusable = focusable;
  }

  public void setImage(Image image)
  {
    m_image = image;
  }

  public Image getImage()
  {
    return m_image;
  }

  public Point computeSize(int wHint, int hHint, boolean changed)
  {
    int w = 0, h = 0;

    if (m_image != null)
    {
      Rectangle rect = m_image.getBounds();
      w = rect.width;
      h = rect.height;
    }

    if (wHint != SWT.DEFAULT)
    {
      w = wHint;
    }

    if (hHint != SWT.DEFAULT)
    {
      h = hHint;
    }

    return new Point(w, h);
  }
}
