/*******************************************************************************
 * Copyright (c) 2011 Oracle America, Inc.  All rights reserved.
 * This source file created for use with the eSWT library is made available
 * under the terms of the Eclipse Public License v1.0.
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/

package org.eclipse.swt.widgets;

import org.eclipse.swt.SWT;

public class WmAdfLabel
  extends Label
{
  protected boolean isFocusable = false;

  public WmAdfLabel(Composite parent, int style)
  {
    super(parent, style);
  }

  public boolean getFocusable()
  {
    return isFocusable;
  }

  public void setFocusable(boolean focusable)
  {
    isFocusable = focusable;
  }

  boolean isTabGroup()
  {
    return false;
  }

  boolean isTabItem()
  {
    if (isFocusable)
      return true;
    return false;
  }
  
  /**
   * Causes the receiver to have the <em>keyboard focus</em>, 
   * such that all keyboard events will be delivered to it.  Focus
   * reassignment will respect applicable platform constraints.
   *
   * @return <code>true</code> if the control got focus, and <code>false</code> if it was unable to.
   *
   * @exception SWTException <ul>
   *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
   *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
   * </ul>
   *
   * @see #forceFocus
   */
  public boolean setFocus() {
	checkWidget();
	if ((internal_style & SWT.NO_FOCUS) != 0) return false;
	return forceFocus();
  }

  public boolean allowTraverseByArrowKey(Event event)
  {
    switch (event.keyCode)
    {
      case SWT.ARROW_UP:
      case SWT.ARROW_LEFT:
      case SWT.ARROW_DOWN:
      case SWT.ARROW_RIGHT:
        return true;
    }
    return false;
  }

  // MDO - copied this from eSWT button

  protected boolean traverse(Event event)
  {
    if (isDisposed())
      return false;
    if (allowTraverseByArrowKey(event))
    {
      return traverseByArrowKey(event);
    }
    return super.traverse(event);
  }

  protected void checkSubclass()
  {
    // super() throws runtime error
  }

  protected boolean isValidSubclass()
  {
    return true;
  }
}
