/*******************************************************************************
 * Copyright (c) 2000, 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

/*******************************************************************************
 * Additions/modifications to this source file by Oracle America, Inc. 2011
 *******************************************************************************/

#ifndef __UGL_BUTTON_H__
#define __UGL_BUTTON_H__

#include "UGL_Error.h"
#include "UGLTypes.h"

#ifdef __cplusplus
extern "C" {
#endif


/**
 * Sets the label of a button.  Please note that a button may not display
 * its image and its label simultaneously.
 *
 * @ingroup Button
 *
 * @param handle the button handle
 * @param string the label
 * @param error the error status of the call
 */
void Button_SetLabel(UGL_Int handle, UGL_String string, UGL_Error error);

/**
 * Set the selection state of a button.
 *
 * @ingroup Button
 *
 * @param handle the button handle
 * @param selection true for selected, false for unselected
 * @param error the error status of the call
 */
void Button_SetSelection(UGL_Int handle, UGL_Boolean selection, UGL_Error error);

/**
 * Gets the selection state of a button.
 *
 * @param handle the button handle
 * @return the selection state of the button
 */
UGL_Boolean Button_GetSelection(UGL_Int handle, UGL_Error error);

/**
 * Creates a new Checkbox widget
 *
 * @ingroup Checkbox
 *
 * @param parentHandle handle of parent widget
 * @param style STYLE_FLAT or 0
 * @param error the error status of the call
 *
 * @return handle of new widget
 */
UGL_Int Checkbox_New(UGL_Int parentHandle, UGL_Int style, UGL_Error error);

/**
 * Create a new PushButton widget
 *
 * @ingroup PushButton
 *
 * @param parentHandle handle of parent widget
 * @param style STYLE_FLAT or 0
 * @param error the error status of the call
 *
 * @return handle of new widget
 */
UGL_Int PushButton_New(UGL_Int parentHandle, UGL_Int style, UGL_Error error);

/**
 * Create a new RadioButton widget
 *
 * @ingroup RadioButton
 *
 * @param parentHandle handle of parent widget
 * @param style STYLE_FLAT or 0
 * @param error the error status of the call
 *
 * @return handle of new widget
 */
UGL_Int RadioButton_New(UGL_Int parentHandle, UGL_Int style, UGL_Error error);



/**
 * Sets the position of the label or image on the button.
 * On left to right locales, STYLE_LEAD is left, and STYLE_TRAIL
 * is right.  Right to left locales reverse this notation.
 *
 * @ingroup Button
 *
 * @param handle the button handle
 * @param alignment OS.STYLE_LEAD, OS.STYLE_CENTER or OS.STYLE_TRAIL
 * @param error the error status of the call
 */
void Button_SetAlignment(UGL_Int handle, UGL_Int alignment, UGL_Error error);

/**
 * Sets the image of a button.  Please note that a button may not display
 * its image and its label simultaneously.
 *
 * @ingroup Button
 *
 * @param handle the button handle
 * @param imageHandle the image handle
 * @param error the error status of the call
 */
void Button_SetImage(UGL_Int handle, UGL_Int imageHandle, UGL_Error error);

/**
 * Create a new ToggleButton widget
 *
 * @ingroup ToggleButton
 *
 * @param parentHandle handle of parent widget
 * @param style STYLE_FLAT or 0
 * @param error the error status of the call
 *
 * @return handle of new widget
 */
UGL_Int ToggleButton_New(UGL_Int parentHandle, UGL_Int style, UGL_Error error);


#ifdef __cplusplus
}
#endif

#endif /* __UGL_BUTTON_H__ */
