/*******************************************************************************
 * Copyright (c) 2000, 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

/*******************************************************************************
 * Additions/modifications to this source file by Oracle America, Inc. 2011
 *******************************************************************************/

#ifndef __KEYUTILS_H__
#define __KEYUTILS_H__

#include <windows.h>
#include "UGL_Constants.h"

/*
* This KeyTable makes it possible to translate control keys to their 
* appropriate UGL mappings. 
* I'm not sure that this is the best place for this, but I can't think of
* anywhere better. I also don't know if any memory management issues are
* presented by this declaration.
*/ 
static const int KeyTableSize = 54;
static const int KeyTable[][2] = {
    
	/* Keyboard and Mouse Masks */
	{ VK_MENU,		UGL_KEYCODE_ALT },
	{ VK_SHIFT,		UGL_KEYCODE_SHIFT },
	{ VK_CONTROL,	UGL_KEYCODE_CONTROL },
//	{ VK_????,		UGL_KEYCODE_COMMAND },

	/* Coppied from SWT but not currently used by us or them */
//	{ VK_LBUTTON,	UGL_KEYCODE_BUTTON1 },
//	{ VK_MBUTTON,	UGL_KEYCODE_BUTTON3 },
//	{ VK_RBUTTON,	UGL_KEYCODE_BUTTON2 },

	
	/* Non-Numeric Keypad Keys */
	{ VK_UP,		UGL_KEYCODE_ARROW_UP },
	{ VK_DOWN,		UGL_KEYCODE_ARROW_DOWN },
	{ VK_LEFT,		UGL_KEYCODE_ARROW_LEFT },
	{ VK_RIGHT,		UGL_KEYCODE_ARROW_RIGHT },
	{ VK_PRIOR,		UGL_KEYCODE_PAGE_UP },
	{ VK_NEXT,		UGL_KEYCODE_PAGE_DOWN },
	{ VK_HOME,		UGL_KEYCODE_HOME },
	{ VK_END,		UGL_KEYCODE_END },
	{ VK_INSERT,	UGL_KEYCODE_INSERT },

	/* Virtual and ASCII Keys */
	{ VK_BACK,		UGL_KEYCODE_BACKSPACE },
//	{ VK_RETURN,	UGL_KEYCODE_CR }, //we don't have a CR, I think this might be ENTER
	{ VK_RETURN,	UGL_KEYCODE_ENTER }, //just a guess
	{ VK_DELETE,	UGL_KEYCODE_DELETE },
	{ VK_ESCAPE,	UGL_KEYCODE_ESCAPE },
//	{ VK_RETURN,	UGL_KEYCODE_LF }, //we don't have a LF....
	{ VK_TAB,		UGL_KEYCODE_TAB },

	/* Function Keys */
	{ VK_F1,		UGL_KEYCODE_F1 },
	{ VK_F2,		UGL_KEYCODE_F2 },
	{ VK_F3,		UGL_KEYCODE_F3 },
	{ VK_F4,		UGL_KEYCODE_F4 },
	{ VK_F5,		UGL_KEYCODE_F5 },
	{ VK_F6,		UGL_KEYCODE_F6 },
	{ VK_F7,		UGL_KEYCODE_F7 },
	{ VK_F8,		UGL_KEYCODE_F8 },
	{ VK_F9,		UGL_KEYCODE_F9 },
	{ VK_F10,		UGL_KEYCODE_F10 },
	{ VK_F11,		UGL_KEYCODE_F11 },
	{ VK_F12,		UGL_KEYCODE_F12 },
	{ VK_F13,		UGL_KEYCODE_F13 },
	{ VK_F14,		UGL_KEYCODE_F14 },
	{ VK_F15,		UGL_KEYCODE_F15 },
	
	/* Numeric Keypad Keys */
	{ VK_MULTIPLY,	UGL_KEYCODE_KEYPAD_MULTIPLY },
	{ VK_ADD,		UGL_KEYCODE_KEYPAD_ADD },
	{ VK_RETURN,	UGL_KEYCODE_KEYPAD_ENTER }, //SWT maps to SWT.KEYPAD_CR
	{ VK_SUBTRACT,	UGL_KEYCODE_KEYPAD_SUBTRACT },
	{ VK_DECIMAL,	UGL_KEYCODE_KEYPAD_DECIMAL },
	{ VK_DIVIDE,	UGL_KEYCODE_KEYPAD_DIVIDE },
	{ VK_NUMPAD0,	UGL_KEYCODE_KEYPAD_0 },
	{ VK_NUMPAD1,	UGL_KEYCODE_KEYPAD_1 },
	{ VK_NUMPAD2,	UGL_KEYCODE_KEYPAD_2 },
	{ VK_NUMPAD3,	UGL_KEYCODE_KEYPAD_3 },
	{ VK_NUMPAD4,	UGL_KEYCODE_KEYPAD_4 },
	{ VK_NUMPAD5,	UGL_KEYCODE_KEYPAD_5 },
	{ VK_NUMPAD6,	UGL_KEYCODE_KEYPAD_6 },
	{ VK_NUMPAD7,	UGL_KEYCODE_KEYPAD_7 },
	{ VK_NUMPAD8,	UGL_KEYCODE_KEYPAD_8 },
	{ VK_NUMPAD9,	UGL_KEYCODE_KEYPAD_9 },
//	{ VK_????,		UGL_KEYPAD_EQUAL }, //Coppied over from SWT just for reference
	
	/* Other Keys */
	{ VK_CAPITAL,	UGL_KEYCODE_LOCK_CAPS },
	{ VK_NUMLOCK,	UGL_KEYCODE_LOCK_NUM },
	{ VK_SCROLL,	UGL_KEYCODE_LOCK_SCROLL },
	{ VK_PAUSE,		UGL_KEYCODE_PAUSE },
	{ VK_CANCEL,	UGL_KEYCODE_END }, //SWT uses SWT.BREAK here
	{ VK_SNAPSHOT,	UGL_KEYCODE_PRINTSCREEN },
//	{ VK_????,		UGL_KEYCODE_HELP }, //Coppied over from SWT just for reference
};

#ifndef _WIN32_WCE
	// no need for extra fixup
#else
/*
 * Taken from http://msdn.microsoft.com/en-us/library/bb431750.aspx
*/ 
#define VK_SHIFT 0x10
#define VK_CTRL 0x11
#define VK_ALT 0x12
#define VK_UP 0x26
#define VK_DOWN 0x28
#define VK_TAB 0x09
#define VK_OEM_1 0xBA
#define VK_ESCAPE 0x1B
#define VK_OEM_PLUS 0xBB
#define VK_OEM_COMMA 0xBC
#define VK_OEM_MINUS 0xBD
#define VK_OEM_PERIOD 0xBE
#define VK_OEM_2 0xBF
#define VK_OEM_3 0xC0
#define VK_OEM_4 0xDB
#define VK_OEM_5 0xDC
#define VK_OEM_6 0xDD
#define VK_OEM_7 0xDE
#define VK_OEM_8 0xDF
#define VK_OEM_102 0xE2

static const int WMKeyTableSize = 12;
static const int WMKeyTable[][2] = {    
	{ VK_OEM_1, ';' },
	{ VK_OEM_PLUS, '+' },
	{ VK_OEM_PLUS, '=' },
	{ VK_OEM_COMMA , ',' },
	{ VK_OEM_MINUS, '-' },
	{ VK_OEM_PERIOD, '.' },
	{ VK_OEM_2, '/' },
	{ VK_OEM_3, '`' },
	{ VK_OEM_4, '[' },
	{ VK_OEM_5, '\\' },
	{ VK_OEM_6, ']' },
	{ VK_OEM_7, '\'' },
};
#endif // #ifndef _WIN32_WCE

/**
 * This method converts the incomming UGL keycode to be a native 
 * VK_* keycode.
 */
int KeyUtils_ConvertToNativeKeycode(int keycode);

#endif /* __KEYUTILS_H__ */
