/*******************************************************************************
 * Copyright (c) 2000, 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

/*******************************************************************************
 * Additions/modifications to this source file by Oracle America, Inc. 2011
 *******************************************************************************/

#include "TypeConverter.h"

const UGL_Char* getNativeStringData(UGL_String string, UGL_Boolean* shouldFree) {
	*shouldFree = FALSE;
	return string;
}

UGL_Int getNativeStringLength(UGL_String string) {
	// make sure null-terminated!
	return lstrlen(string);
}

UGL_String convertToNativeString(const UGL_Char* data, UGL_Int length, UGL_Boolean* shouldFree) {
	if (length >= 0) {
		UGL_String string = (UGL_String)malloc((length + 1) * sizeof(UGL_Char));
		if (string) {
			wcsncpy(string, data, length);
			string[length] = 0;
			*shouldFree = UGL_TRUE;
			return string;
		}
	}

	*shouldFree = UGL_FALSE;
	return (UGL_String)NULL;
}

void freeNativeStringData(const UGL_Char* data) {
	// Do nothing since we never return TRUE for freeing string data
}

void freeNativeString(UGL_String string) {
	free((void *)string);
}

void freeNativeStringArray(UGL_StringArray stringArray) {
	// Each of the string elements needs to be free'd as well
	int index=0;
	while (stringArray[index] != NULL) {
		free(stringArray[index]);
		index++;
	}

	free((void *)stringArray);
}

void freeNativeIntArray(UGL_IntArray intArray) {
	free((void *)intArray);
}

void freeNativeByteArray(UGL_ByteArray array) {
	free((void *)array);
}

UGL_Int getNativeStringArrayLength(UGL_StringArray nativeArray) {
	UGL_Int length = 0;
	if (nativeArray == NULL) return 0;
	while (nativeArray[length] != NULL) {length++;}
	return length;
}

UGL_String getNativeStringArrayElement(UGL_StringArray nativeArray, UGL_Int index, UGL_Boolean* shouldFreeResult) {	
	*shouldFreeResult = FALSE;
	if (nativeArray == NULL) return NULL;
	return nativeArray[index];
}
