/*******************************************************************************
 * Copyright (c) 2000, 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Radoslav Gerganov <r.gerganov@prosyst.bg> - bug 213627
 *******************************************************************************/

/*******************************************************************************
 * Additions/modifications to this source file by Oracle America, Inc. 2011
 *******************************************************************************/

#ifndef __UGL_UGL_CONSTANTS_H__
#define __UGL_UGL_CONSTANTS_H__

#ifdef __cplusplus
extern "C" {
#endif


/** Control background color id */
#define UGL_COLOR_CONTROL_BACKGROUND 17

/** Control foreground color id */
#define UGL_COLOR_CONTROL_FOREGROUND 18

/** Control normal highlight color id */
#define UGL_COLOR_CONTROL_HIGHLIGHT_NORMAL 19

/** Control dark shadow color id */
#define UGL_COLOR_CONTROL_SHADOW_DARK 22

/** Control normal shadow color id */
#define UGL_COLOR_CONTROL_SHADOW_NORMAL 21

/** Tool tip background color id */
#define UGL_COLOR_INFO_BACKGROUND 24

/** Tool tip foreground color id */
#define UGL_COLOR_INFO_FOREGROUND 25

/** Text widget highlight background color id */
#define UGL_COLOR_TEXT_HIGHLIGHT_BACKGROUND 14

/** Text widget highlight foreground color id */
#define UGL_COLOR_TEXT_HIGHLIGHT_FOREGROUND 15

/** Invalid image error */
#define UGL_ERROR_INVALID_IMAGE -8

/** I/O error */
#define UGL_ERROR_IO -7

/** No error has occurred. */
#define UGL_ERROR_NONE 0

/** API/function is unimplemented */
#define UGL_ERROR_NOT_IMPLEMENTED -1

/** 
 * An unrecoverable error occurred. The error is not specified further.
 * Any API may throw this error.
 */
#define UGL_ERROR_OTHER -3

/** Out of memory/handles. Any API may throw this error. */
#define UGL_ERROR_OUT_OF_MEMORY -4

/** Attempt to load/create an image with unsupported color depth. */
#define UGL_ERROR_UNSUPPORTED_DEPTH -5

/** Attempt to load/save an image of an unsupported format. */
#define UGL_ERROR_UNSUPPORTED_FORMAT -6

/** Focus event triggered when a Control has gained focus. */
#define UGL_EVENT_FOCUS_IN 1

/** Focus event triggered when a Control has lost focus. */
#define UGL_EVENT_FOCUS_OUT 2

/** Focus event flag which indicates that the focus change was temporary. */
#define UGL_EVENT_FOCUS_TEMPORARY_FLAG (1 << 16)

/** Key down event. */
#define UGL_EVENT_KEY_DOWN 1

/** Key up event. */
#define UGL_EVENT_KEY_UP 2

/** The pointer has been double clicked. */
#define UGL_EVENT_POINTER_DOUBLECLICK 7

/** The pointer has been pressed. */
#define UGL_EVENT_POINTER_DOWN 1

/** The pointer has been released. */
#define UGL_EVENT_POINTER_DRAG 6

/** The pointer has entered a control's bounds. */
#define UGL_EVENT_POINTER_ENTER 4

/** The pointer has left a control's bounds. */
#define UGL_EVENT_POINTER_EXIT 5

/** The pointer has moved. */
#define UGL_EVENT_POINTER_MOVE 3

/** The pointer has been released. */
#define UGL_EVENT_POINTER_UP 2

/** 
 * Event code that will be passed into the shell callback method
 * when the shell is activated.
 */
#define UGL_EVENT_SHELL_ACTIVATED 205

/** 
 * Event code that will be passed into the shell callback method
 * when the user clicks on the close button.
 */
#define UGL_EVENT_SHELL_CLOSING 201

/** 
 * Event code that will be passed into the shell callback method
 * when the shell is deactivated.
 */
#define UGL_EVENT_SHELL_DEACTIVATED 206

/** 
 * Event code that will be passed into the shell callback method
 * when the shell is deiconified.
 */
#define UGL_EVENT_SHELL_DEICONIFIED 204

/** 
 * Event code that will be passed into the shell callback method
 * when the shell is iconified.
 */
#define UGL_EVENT_SHELL_ICONIFIED 203

/** Font style flag bold */
#define UGL_FONT_STYLE_BOLD (1 << 1)

/** Font style flag italic */
#define UGL_FONT_STYLE_ITALIC (1 << 2)

/** Font style flag underline */
#define UGL_FONT_STYLE_UNDERLINED (1 << 3)

#define UGL_KEYCODE_ALT 1018

#define UGL_KEYCODE_ARROW_DOWN 1016

#define UGL_KEYCODE_ARROW_LEFT 1013

#define UGL_KEYCODE_ARROW_RIGHT 1014

#define UGL_KEYCODE_ARROW_UP 1015

#define UGL_KEYCODE_BACKSPACE 1000

#define UGL_KEYCODE_CONTROL 1019

#define UGL_KEYCODE_DELETE 1012

#define UGL_KEYCODE_DEVICE0 1063

#define UGL_KEYCODE_DEVICE1 1064

#define UGL_KEYCODE_DEVICE2 1065

#define UGL_KEYCODE_DEVICE3 1066

#define UGL_KEYCODE_END 1008

#define UGL_KEYCODE_ENTER 1002

#define UGL_KEYCODE_ESCAPE 1003

#define UGL_KEYCODE_F1 1023

#define UGL_KEYCODE_F10 1032

#define UGL_KEYCODE_F11 1033

#define UGL_KEYCODE_F12 1034

#define UGL_KEYCODE_F13 1035

#define UGL_KEYCODE_F14 1036

#define UGL_KEYCODE_F15 1037

#define UGL_KEYCODE_F16 1038

#define UGL_KEYCODE_F17 1039

#define UGL_KEYCODE_F18 1040

#define UGL_KEYCODE_F19 1041

#define UGL_KEYCODE_F2 1024

#define UGL_KEYCODE_F20 1042

#define UGL_KEYCODE_F21 1043

#define UGL_KEYCODE_F22 1044

#define UGL_KEYCODE_F23 1045

#define UGL_KEYCODE_F24 1046

#define UGL_KEYCODE_F3 1025

#define UGL_KEYCODE_F4 1026

#define UGL_KEYCODE_F5 1027

#define UGL_KEYCODE_F6 1028

#define UGL_KEYCODE_F7 1029

#define UGL_KEYCODE_F8 1030

#define UGL_KEYCODE_F9 1031

#define UGL_KEYCODE_HELP 1062

#define UGL_KEYCODE_HOME 1007

#define UGL_KEYCODE_INSERT 1011

/** 
 * Keyboard event constant representing the numeric key
 * pad zero key.
 */
#define UGL_KEYCODE_KEYPAD_0 1072

/** 
 * Keyboard event constant representing the numeric key
 * pad one key.
 */
#define UGL_KEYCODE_KEYPAD_1 1073

/** 
 * Keyboard event constant representing the numeric key
 * pad two key.
 */
#define UGL_KEYCODE_KEYPAD_2 1074

/** 
 * Keyboard event constant representing the numeric key
 * pad three key.
 */
#define UGL_KEYCODE_KEYPAD_3 1075

/** 
 * Keyboard event constant representing the numeric key
 * pad four key.
 */
#define UGL_KEYCODE_KEYPAD_4 1076

/** 
 * Keyboard event constant representing the numeric key
 * pad five key.
 */
#define UGL_KEYCODE_KEYPAD_5 1077

/** 
 * Keyboard event constant representing the numeric key
 * pad six key.
 */
#define UGL_KEYCODE_KEYPAD_6 1078

/** 
 * Keyboard event constant representing the numeric key
 * pad seven key.
 */
#define UGL_KEYCODE_KEYPAD_7 1079

/** 
 * Keyboard event constant representing the numeric key
 * pad eight key.
 */
#define UGL_KEYCODE_KEYPAD_8 1080

/** 
 * Keyboard event constant representing the numeric key
 * pad nine key.
 */
#define UGL_KEYCODE_KEYPAD_9 1081

/** 
 * Keyboard event constant representing the numeric key
 * pad add key.
 */
#define UGL_KEYCODE_KEYPAD_ADD 1068

/** 
 * Keyboard event constant representing the numeric key
 * pad decimal key.
 */
#define UGL_KEYCODE_KEYPAD_DECIMAL 1070

/** 
 * Keyboard event constant representing the numeric key
 * pad divide key.
 */
#define UGL_KEYCODE_KEYPAD_DIVIDE 1071

/** 
 * Keyboard event constant representing the numeric key
 * pad enter key.
 */
#define UGL_KEYCODE_KEYPAD_ENTER 1082

/** 
 * Keyboard event constant representing the numeric key
 * pad multiply key.
 */
#define UGL_KEYCODE_KEYPAD_MULTIPLY 1067

/** 
 * Keyboard event constant representing the numeric key
 * pad subtract key.
 */
#define UGL_KEYCODE_KEYPAD_SUBTRACT 1069

#define UGL_KEYCODE_LOCK_CAPS 1020

#define UGL_KEYCODE_LOCK_NUM 1021

#define UGL_KEYCODE_LOCK_SCROLL 1022

#define UGL_KEYCODE_PAGE_DOWN 1010

#define UGL_KEYCODE_PAGE_UP 1009

#define UGL_KEYCODE_PAUSE 1006

#define UGL_KEYCODE_PRINTSCREEN 1005

#define UGL_KEYCODE_SHIFT 1017

#define UGL_KEYCODE_TAB 1001

#define UGL_MODIFIER_MASK_ALT 1

#define UGL_MODIFIER_MASK_CONTROL 4

#define UGL_MODIFIER_MASK_SHIFT 2

#define UGL_POINTER_MASK_BUTTON1 8

#define UGL_POINTER_MASK_BUTTON2 16

#define UGL_POINTER_MASK_BUTTON3 32

/** 
 * Pointer button masks must not conflict with the alt, control,
 * and shift modifiers
 */
#define UGL_POINTER_MASK_NOBUTTON 0

#define UGL_POINTER_POPUP_TRIGGER (1 << 6)

/** Interpret tab characters (<code>"\t"</code>) as tab stops. */
#define UGL_STRING_EXPAND_TAB (1 << 1)

/** 
 * Interpret a mnemonic character (<code>&amp;</code>) to
 * underline the next character	instead of rendering the
 * ampersand.
 */
#define UGL_STRING_MNEMONIC (1 << 2)

/** Render text transparently on the background. */
#define UGL_STRING_TRANSPARENT (1 << 3)

/** 
 * Wrap text at the line delimiters. Valid line delimiters are
 * <code>"\r"</code>, <code>"\n"</code> and <code>"\r\n"</code>
 */
#define UGL_STRING_WRAP 1

/** 
 * Shell style indicating that while the Shell is open it will
 * block input to every other Shell in the application.  Modality
 * styles are treated as hints since typically all types of modality
 * are not supported on every platform.
 */
#define UGL_STYLE_APPLICATION_MODAL (1 << 6)

/** Border */
#define UGL_STYLE_BORDER (1 << 11)

/** Center alignment */
#define UGL_STYLE_CENTER (1 << 1)

/** The MenuItem style for a checkbox item */
#define UGL_STYLE_CHECK (1 << 5)

/** Close trim */
#define UGL_STYLE_CLOSE 1

/** Drop-down menu and combo behavior */
#define UGL_STYLE_DROP_DOWN (1 << 26)

/** Flat */
#define UGL_STYLE_FLAT (1 << 12)

/** Horizontal style */
#define UGL_STYLE_HORIZONTAL (1 << 4)

/** Horizontal scroll bars */
#define UGL_STYLE_H_SCROLL (1 << 16)

/** Lead alignment (left alignment in left-to-right languages) */
#define UGL_STYLE_LEAD 1

/** Left to right orientation */
#define UGL_STYLE_LEFT_TO_RIGHT (1 << 14)

/** Maximize trim */
#define UGL_STYLE_MAX (1 << 2)

/** Minimize trim */
#define UGL_STYLE_MIN (1 << 1)

/** Modeless */
#define UGL_STYLE_MODELESS (1 << 7)

/** Multiple selection and multiple line style */
#define UGL_STYLE_MULTI (1 << 23)

/** Do not clear the background on paint events */
#define UGL_STYLE_NO_BACKGROUND (1 << 18)

/** Do not take focus */
#define UGL_STYLE_NO_FOCUS (1 << 20)

/** Do not merge paint events */
#define UGL_STYLE_NO_MERGE_PAINTS (1 << 19)

/** Do not redraw the entire client area on resize */
#define UGL_STYLE_NO_REDRAW_RESIZE (1 << 21)

/** No trimmings */
#define UGL_STYLE_NO_TRIM (1 << 3)

/** 
 * Shell style indicating that while the Shell is open it will
 * block input to its parent.  Modality styles are treated as
 * hints since typically all types of modality are not supported
 * on every platform.
 */
#define UGL_STYLE_PRIMARY_MODAL (1 << 8)

/** The MenuItem style for a radio button item */
#define UGL_STYLE_RADIO (1 << 4)

/** Read-only text field */
#define UGL_STYLE_READ_ONLY (1 << 24)

/** Right to left orientation */
#define UGL_STYLE_RIGHT_TO_LEFT (1 << 15)

/** Shadow inside */
#define UGL_STYLE_SHADOW_IN (1 << 5)

/** No shadow */
#define UGL_STYLE_SHADOW_NONE (1 << 7)

/** Shadow outside */
#define UGL_STYLE_SHADOW_OUT (1 << 6)

/** Title trim */
#define UGL_STYLE_TITLE (1 << 5)

/** Trail alignment (right alignment in left-to-right languages) */
#define UGL_STYLE_TRAIL (1 << 2)

/** Vertical style */
#define UGL_STYLE_VERTICAL (1 << 3)

/** Vertical scroll bars */
#define UGL_STYLE_V_SCROLL (1 << 17)

/** Line wrapping text */
#define UGL_STYLE_WRAP (1 << 8)

/** virtual */
#define UGL_STYLE_VIRTUAL (1 << 28)


/** 
 * The constant used to ask the platform whether it supports
 * making child shells user resizable.  The return value from
 * the platform should be non-zero if the platform can support
 * child shells that are user resizable.
 */
#define UGL_CAPABILITY_SHELL_CHILD_RESIZABLE 2

/** 
 * The constant used to ask the platform whether it supports minimizing
 * or maximizing shells.  The return value from the platform should be
 * non-zero if the platform supports minimizing and maximizing shells.
 */
#define UGL_CAPABILITY_SHELL_MINMAX 3

/** 
 * The constant used to ask the platform whether it supports
 * making toplevel shells user resizable.  The return value from
 * the platform should be non-zero if the platform can support
 * toplevel shells that are user resizable.
 */
#define UGL_CAPABILITY_SHELL_TOPLEVEL_RESIZABLE 1

/** Inactive caption background color id */
#define UGL_COLOR_INACTIVE_CAPTION_BACKGROUND 4

/** Inactive caption foreground color id */
#define UGL_COLOR_INACTIVE_CAPTION_FOREGROUND 5

#define UGL_INFO_HASMENU_MAYBE 0

#define UGL_INFO_HASMENU_NO 2

#define UGL_INFO_HASMENU_YES 1

/** Label style for vertically centering the text. */
#define UGL_STYLE_MIDDLE (1 << 1)

/** File dialog style for opening a file. */
#define UGL_STYLE_OPEN (1 << 12)

/** File dialog style for saving a file. */
#define UGL_STYLE_SAVE (1 << 13)

/** Label style for top aligning the text. */
#define UGL_STYLE_TOP 1



/** 
 * List event which indicates that the currently selected item has
 * been actioned.
 */
#define UGL_EVENT_LIST_ACTION 2

/** List event which indicates that a new item has been selected. */
#define UGL_EVENT_LIST_SELECTION 1

#define UGL_SCROLLBAR_ACTION_BLOCK_DECREMENT 2

#define UGL_SCROLLBAR_ACTION_BLOCK_INCREMENT 3

#define UGL_SCROLLBAR_ACTION_END 6

#define UGL_SCROLLBAR_ACTION_HOME 5

#define UGL_SCROLLBAR_ACTION_TRACK 4

#define UGL_SCROLLBAR_ACTION_UNIT_DECREMENT 0

#define UGL_SCROLLBAR_ACTION_UNIT_INCREMENT 1

#define UGL_SCROLLBAR_ACTION_NONE 7



/** Title bar background color id */
#define UGL_COLOR_TITLE_BACKGROUND 26

/** Title bar foreground color id */
#define UGL_COLOR_TITLE_FOREGROUND 27

/** 
 * Combo was default selected when the user pressed the ENTER key.
 * Passed as an argument to the method registered as the CALLBACK_COMBO.
 */
#define UGL_EVENT_COMBO_DEFAULTSELECTION 2

/** 
 * Combo text changed.  Passed as an argument to the
 * method registered as the CALLBACK_COMBO.
 */
#define UGL_EVENT_COMBO_TEXT 1

/** 
 * Event indicating that a menu that has been popped up or
 * dropped down has been hidden (i.e., no longer displayed).
 */
#define UGL_EVENT_MENU_HIDE 2

/** Event indicating that a menu has been popped up or dropped down. */
#define UGL_EVENT_MENU_SHOW 1

#define UGL_LINE_DASH (1 << 1)

#define UGL_LINE_DASHDOT (1 << 3)

#define UGL_LINE_DASHDOTDOT (1 << 4)

#define UGL_LINE_DOT (1 << 2)

#define UGL_LINE_SOLID 1

#define UGL_MESSAGEBOX_CANCEL (1 << 12)

#define UGL_MESSAGEBOX_ICON_ERROR (1 << 18)

#define UGL_MESSAGEBOX_ICON_INFORMATION (1 << 19)

#define UGL_MESSAGEBOX_ICON_QUESTION (1 << 20)

#define UGL_MESSAGEBOX_ICON_WARNING (1 << 21)

#define UGL_MESSAGEBOX_ICON_WORKING (1 << 22)

#define UGL_MESSAGEBOX_NO (1 << 14)

#define UGL_MESSAGEBOX_OK (1 << 11)

#define UGL_MESSAGEBOX_RETRY (1 << 15)

#define UGL_MESSAGEBOX_YES (1 << 13)

/** 
 * ProgressBar style for indicating that the amount
 * of work to be done is not known.
 */
#define UGL_STYLE_INDETERMINATE (1 << 1)

/** 
 * Shell style for indicating that a Shell is
 * to remain on top of the z order.  This style
 * is a hint, so it may not be implemented everywhere.
 */
#define UGL_STYLE_ON_TOP (1 << 25)

/** 
 * A hint the ProgressBar widget to display as
 * a constant line.
 */
#define UGL_STYLE_SMOOTH (1 << 16)

/** 
 * Shell style indicating that while the Shell is open it will
 * block all input from the system. Modality styles are treated
 * as hints since all types of modality are not supported on every
 * platform.
 */
#define UGL_STYLE_SYSTEM_MODAL (1 << 27)

/**
 * System icon for error.
 */
#define UGL_IMAGE_ERROR 1

/**
 * System icon for information.
 */
#define UGL_IMAGE_INFORMATION 2

/**
 * System icon for question.
 */
#define UGL_IMAGE_QUESTION 3

/**
 * System icon for warning.
 */
#define UGL_IMAGE_WARNING 4

#ifdef __cplusplus
}
#endif

#endif /* __UGL_UGL_CONSTANTS_H__ */
