/*******************************************************************************
 * Copyright (c) 2000, 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

/*******************************************************************************
 * Additions/modifications to this source file by Oracle America, Inc. 2011
 *******************************************************************************/

#pragma once
//========================================================================================
#include "Callback.h"
#include "UGL_Error.h"
//========================================================================================

#define WINDOW_CLASS_NAME		_T("Window Class Name\0")
#define UGL_DEVICE(handle)		((PUGL_Device)handle)
#define UGL_DISPLAY(handle)		((PUGL_Display)handle)

/* creation constants used to distinguish between
 * creating a shell and any other control. 
 */
#define UI_CREATE_SHELL			(WM_APP + 1)
#define UI_CREATE_CONTROL		(WM_APP + 2)
#define UI_DESTROY_WINDOW 		(WM_APP + 3) 
/*
 * SetFocus(HWND) requires that it be called from the same thread
 * that the control was created from. 
 * Therefore, we have to create our own message to send to the right
 * thread.
 */
#define UI_SET_FOCUS			(WM_APP + 4)
#define UI_MOVE_ABOVE			(WM_APP + 5)
#define UI_SET_VISIBLE			(WM_APP + 6)
#define UI_COMMAND_CHILD		(WM_APP + 7)
#define UI_GET_MINIMUM_SIZE		(WM_APP + 8)
#define UI_SET_BOUNDS			(WM_APP + 9)
#define UI_SCROLLBAR_ORIENTATION (WM_APP + 11)
#define UI_SCROLLABLE_GETINSETS	 (WM_APP + 12)	
#ifdef _WIN32_WCE
#define	UI_ADJUST_CHILD_BOUNDS	 (WM_APP + 13)
#endif

#define UI_CREATE_MENUBAR		(WM_APP + 14)
#define UI_GETBACKGROUNDRGB		(WM_APP + 15)
#define UI_SET_ENABLED			(WM_APP + 16)
#define UI_SETMULTIPLE			(WM_APP + 17)
#define UI_DRAWITEM				(WM_APP + 18)
#define UI_CREATE_POPUP			(WM_APP + 19)
#define UI_SHOW_POPUP			(WM_APP + 20)

#ifdef EANBLE_KEYBOARD_SCROLLING
#define UI_SCROLL_KEY			(WM_APP + 21)	// Passed to container classes when one of the 
#endif
                                                // child widgets gets an arrow keypress
#define UI_GETDC				(WM_APP + 22)

// This message is sent to child widgets when a WM_NOTIFY
// is recieved in the parent.  The wParam and lParam are
// not modifed at all before sending.
#define UI_CHILD_NOTIFY			(WM_APP + 23)

#define UI_GETFOREGROUNDRGB		(WM_APP + 24)
#define UI_SETBACKGROUND		(WM_APP + 25)
#define UI_SETFOREGROUND		(WM_APP + 26)
#define UI_SETFONT				(WM_APP + 27)

#define UI_SETCALLBACKTARGET	(WM_APP + 28)

#define UI_NOTIFY_TRAY			(WM_APP + 29)

struct UGL_Control;
struct UGL_Font;
struct UGL_MenuItem;

typedef struct ControlList {
	struct UGL_Control* control;
	struct ControlList* next;
} ControlList, *PControlList;

typedef struct MenuList {
	UGL_Int menuHandle;
	HMENU hMenu;
	struct MenuList* next;
} MenuList, *PMenuList;
//========================================================================================

typedef void (*DEVICE_DISPOSE)(UGL_Int handle, UGL_Error uglError);

typedef struct UGL_Device {
	DEVICE_DISPOSE device_dispose;
} UGL_Device, *PUGL_Device;
//========================================================================================

typedef enum {DRAG_START, DRAGGING} DragState;

typedef struct UGL_Display {
	UGL_Device ugl_device;

	CallbackLookup *callback_lookup;
	int threadID;
	int processID;
	HINSTANCE hInstance;
	HWND creation_hWnd;

	struct UGL_Tray *tray;
	
	struct UGL_MenuItem **menuItems;
	int menuItemsLength;
	// new for softkey. Because menuItemsLength does not decrement when menuitem is removed
	int menuItemCount;

	// Cache the control default font because it is
	// requested alot and needs to be cleaned up 
	// at the end.  This font is not the same as the
	// Device_GetSystemFont return value because the
	// return value for that font is diposed in Java.
	struct UGL_Font* defaultControlFont;

	/**
	 * Used to keep track of KeyEvent data in Display.c
	 */
	int lastKey;
	int lastAscii;
	BOOL lastVirtual;
	BOOL lastNull;
	BOOL lastDead;
	BOOL isEmulator;

	// Fields used to track the current mouse drag state
	HWND dragWindow; // The window being dragged.  If this value is NULL
	                 // then the other two fields are meaningless
	RECT dragZone;   // The area the mouse has to move out of to
	                 // start a drag. The values are relative to the dragWindow
	UINT dragStartTime; // The time where the drag starts, even if the mouse
	                    // has not left the drag zone
	DragState dragState; // The current dragging state. 
	

	HWND lastMouseWind; //last window with a MouseMove Event

	// These two fields should not be referenced by any method outside 
	// of Display_AddModalShell, Display_RemoveModalShell, Display_AddShell,
	// Display_RemoveShell and Display_EnableShell.
	PControlList modalShells;   // Tracks the open modal dialogs
	PControlList enabledShells; // Tracks the shells which java thinks
	                            // are enabled.

	// Place holder for expanded eSWT data.
	void *expandedData;

	// common control dll version numbers
	int comCtrl32Major;
	int comCtrl32Minor;

	// Tracks defined menus.
	PMenuList menus;
	// need to track active menu for HIDE event handling
	struct UGL_Menu * activeMenu; 

	// need to track active menu bar
	struct UGL_Menu * activeMenuBar; 

	// need to track active menu for active Shell
	struct UGL_Control * activeShell; 


} UGL_Display, *PUGL_Display;
//========================================================================================

/**
 * Used for Device_GetTypefaces() only 
 */
typedef struct UGL_EnumFont {
	int scalable;
	UGL_StringArray results;
	int result_count;
	int capacity;
	BOOL variants;
} UGL_EnumFont, *PUGL_EnumFont;

struct UGL_Font* _Device_GetSystemFont(UGL_Error uglError);

/**
 * When a modal shell is added to the display all other visible shell's are disabled. When 
 * the last modal shell is removed from the display all shells that want to be enabled
 * will be re-enabled
 */

void Display_UpdateModalShellVisibility(PUGL_Display display, struct UGL_Control* modalShell, BOOL visible, UGL_Error error);

/**
 * This method will track the new shell and manage it's enabled state
 * based on the current modal dialog.
 */
void Display_AddShell(PUGL_Display display, struct UGL_Control* shell, UGL_Error error);

/**
 * Stops tracking the specified shell.  This should only be called 
 * when a shell is being disposed.
 */
void Display_RemoveShell(PUGL_Display display, struct UGL_Control* shell);

/**
 * Returns TRUE if the shell needs to be updated with the new
 * enabled status.  It will return FALSE if the shell already has the 
 * style specified.  For example, If the this method is called to enable a shell while
 * a modal dialog is visible it will return FALSE, since the shell must remain 
 * disabled while the dialog is visible, but when the dialog is hidden the desired
 * enabled state will be remembered and updated.
 */
BOOL Display_EnableShell(PUGL_Display display, struct UGL_Control* shell, BOOL enable);

void Display_AddMenuItem(struct UGL_MenuItem *menuItem);

struct UGL_MenuItem * Display_GetMenuItem(PUGL_Display display, int id);

void Display_RemoveMenuItem(struct UGL_MenuItem *menuItem);

#ifdef _WIN32_WCE

//struct UGL_MenuItem * Display_FindMenuItem(PUGL_Display display, int index);
struct UGL_MenuItem * Display_FindMenuItem(PUGL_Display display, HWND hwndCB, int index);

//void Display_DecMenuItemIndex(PUGL_Display display, int start);
void Display_DecMenuItemIndex(PUGL_Display display, HWND hwndCB, int start);

//void Display_IncMenuItemIndex(PUGL_Display display, int start);
void Display_IncMenuItemIndex(PUGL_Display display, HWND hwndCB, int start);

#endif

int _Display_GetComCtrlMajorVersion(PUGL_Display display, UGL_Error uglError);
//========================================================================================
