/*******************************************************************************
 * Copyright (c) 2000, 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

#include "ugl_win32.h"

#if _WIN32_WCE 

#if (_WIN32_WCE >= 420)
#include <aygshell.h>
#endif /* _WIN32_WCE >= 420 */

#if !(WIN32_PLATFORM_PSPC) && !(WIN32_PLATFORM_WFSP)  
#include "Commctrl.h"
#endif /* !(WIN32_PLATFORM_PSPC) && !(WIN32_PLATFORM_WFSP) */

#endif /* _WIN32_WCE */

#include "Image.h"
#include "Ugl_Win32_DC.h"
#include "UGL_Win32_Menu.h"
#include "Graphics.h"
#include "Widget.h"
#include "UGL_Win32_Widget.h"

#include "Control.h"
#include "Shell.h"
#include "Menu.h"

#ifdef _WIN32_WCE
#define SHELL_WINDOW_CLASS _T("Dialog")
#else
#define SHELL_WINDOW_CLASS _T("#32770")
#endif

static UGL_Int _Shell_ExStyle(UGL_Int ugl_style, BOOL isChild);
static UGL_Int _Shell_Style(UGL_Int ugl_style, BOOL isChild);
static UGL_Int _Shell_New(UGL_Int parentHandle, UGL_Int displayHandle, UGL_Int ugl_style, UGL_Error uglError);
static void _Shell_MoveAbove(PUGL_Control control, PUGL_Control sibling, UGL_Error error);
static UGL_Boolean _Shell_GetMaximized(PUGL_Shell shell);

void _Shell_GetMinimumSize(UGL_Int handle, POINT *point, UGL_Error uglError);
void _Dispose_Shell(UGL_Int handle, UGL_Error uglError);
static LRESULT CALLBACK _Shell_Proc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
//========================================================================================

UGL_Int Shell_New(UGL_Int displayHandle, UGL_Int ugl_style, UGL_Error uglError) {
	return _Shell_New(0, displayHandle, ugl_style, uglError);
}

//========================================================================================

#if defined(WIN32_PLATFORM_PSPC)
/**
 * Ensures that the menubar associated with the specified shell
 * is visible to the user.
 *
 * PocketPC menubars are floating top-level windows which
 * need to be shown and hidden in conjuntion with showing
 * and hiding their owning window.  
 */
void _Shell_EnsureMenuBarIsVisible(UGL_Shell* shell, UGL_Control* parent) {

	int i;
	BOOL hasSubMenu, useSoftkey;
	PUGL_Display display = UGL_WIDGET(shell)->display;
	hasSubMenu = FALSE;
	useSoftkey = FALSE;

#if defined (WIN32_PLATFORM_PSPC)
	useSoftkey = TRUE;
#endif

	for(i = 0; i<display->menuItemsLength; i++){
		if(display->menuItems[i] != NULL && display->menuItems[i]->parent_menu->hwndCB != 0){
			if(display->menuItems[i]->hasSubMenu){
				hasSubMenu = TRUE;
			}
		}
	}
	if(parent == NULL){	
		UGL_Menu* menubar = shell->ugl_menu == NULL ? shell->emptyMenu : shell->ugl_menu;
		// Show the new menubar
		// new for softkey, check device Major version
		if(Platform_GetMajorVersion() >= 5 && useSoftkey){
			if(UGL_WIDGET(shell)->display->menuItemCount>2 || hasSubMenu){
				ShowWindow(menubar->hwndMB, SW_SHOW);
			} else {	
				ShowWindow(menubar->hwndCB, SW_SHOW); 
			}
		} else {
			ShowWindow(menubar->hwndCB, SW_SHOW); 
		}

		if (_Shell_GetMaximized(shell)) Shell_SetMaximized((UGL_Int)shell, TRUE, NULL);
	} else {

		if(shell->ugl_menu != NULL)
		{
			UGL_Menu* menubar = shell->ugl_menu;
		// new for softkey, check device Major version
		if(Platform_GetMajorVersion() >= 5 && useSoftkey){
			if(UGL_WIDGET(shell)->display->menuItemCount>2 || hasSubMenu){
				ShowWindow(menubar->hwndMB, SW_SHOW);
			} else {	
				ShowWindow(menubar->hwndCB, SW_SHOW); 
			}
		} else {
			ShowWindow(menubar->hwndCB, SW_SHOW); 
		}
			

		if (_Shell_GetMaximized(shell)) Shell_SetMaximized((UGL_Int)shell, TRUE, NULL);
		}
	}
	
}
#endif /* WIN32_PLATFORM_PSPC */

//========================================================================================

void Shell_SetText(UGL_Int handle, UGL_String text, UGL_Error uglError) {
	PUGL_Control shell = (PUGL_Control)handle;
	HWND hWnd = shell->hWnd;

	// assumes text is correctly formed
	SetWindowText(hWnd, text);
}
//========================================================================================

void Shell_SetMaximized(UGL_Int handle, UGL_Boolean maximized, UGL_Error uglError) {
	PUGL_Control control = (PUGL_Control)handle;
	HWND hWnd = control->hWnd;
	RECT rect;
	int width, height;
	int flags;


#ifdef _WIN32_WCE
	/*
	* Note: WinCE does not support SW_SHOWMAXIMIZED and SW_RESTORE. The
	* workaround is to resize the window to fit the parent client area.
	*/
	PUGL_Shell shell = UGL_SHELL(handle);
	if (maximized) {
		if(shell->state == MINIMIZED){
			ShowWindow(hWnd, SW_RESTORE);
		}
		ShowWindow(hWnd, SW_MAXIMIZE);
		shell->state = MAXIMIZED;		

#if defined(WIN32_PLATFORM_PSPC)
		SystemParametersInfo(SPI_GETWORKAREA, 0, &rect, 0);
		width = rect.right - rect.left;
		height = rect.bottom - rect.top;
		{
		UGL_Shell* shell = UGL_SHELL(control);
		/* Leave space for the menu bar */
		if ((shell->ugl_menu != NULL) || (shell->emptyMenu != NULL)) {
			HWND hwndCB = shell->ugl_menu != NULL ? shell->ugl_menu->hwndCB : shell->emptyMenu->hwndCB;
			RECT rectCB;

			GetWindowRect(hwndCB, &rectCB);
			height -= rectCB.bottom - rectCB.top;
		}
		}


		if ((control->widget.ugl_style & (UGL_STYLE_BORDER)) == UGL_STYLE_BORDER) { 
			LONG styleBits = GetWindowLong(hWnd, GWL_STYLE) & ~WS_BORDER;
			SetWindowLong(hWnd, GWL_STYLE, styleBits);

			styleBits = GetWindowLong(hWnd, GWL_EXSTYLE) & ~(WS_EX_OVERLAPPEDWINDOW | WS_EX_STATICEDGE | WS_EX_CLIENTEDGE);
			SetWindowLong(hWnd, GWL_EXSTYLE, styleBits);
		}

		flags = SWP_NOZORDER | SWP_FRAMECHANGED | SWP_NOACTIVATE;
		_SetWindowPos(hWnd, NULL, rect.left, rect.top, width, height, flags);	
#endif // is PPC
	} 
	// else if normal
	else{
		shell->state = NORMAL;
		ShowWindow(hWnd, SW_RESTORE);
	}

#else
	int swFlags = maximized ? SW_SHOWMAXIMIZED : SW_RESTORE;

	UGL_SHELL(handle)->state = maximized ? MAXIMIZED : NORMAL;

	if (!IsWindowVisible(hWnd)) return;
	if (maximized == IsZoomed(hWnd)) return;
		
	ShowWindow (hWnd, swFlags);
	UpdateWindow(hWnd);

#endif
}
//========================================================================================

void Shell_Close(UGL_Int shellHandle, UGL_Error uglError) {
	// TODO: remove this method when api is gone
//	PUGL_Control ugl_widget = (PUGL_Control)shellHandle;

//	PostMessage(ugl_widget->hWnd, WM_CLOSE, 0, 0);
}
//========================================================================================


//========================================================================================

UGL_Int _Shell_ExStyle(UGL_Int ugl_style, BOOL isChild) {
	int style = 0;

#ifndef _WIN32_WCE	
	
	//if (IS_FLAG_SET(ugl_style, UGL_STYLE_BORDER)) SET_FLAG(style, WS_EX_CLIENTEDGE);
		
	/*
	* Feature in Windows NT.  When CreateWindowEx() is called with
	* WS_EX_LAYOUTRTL or WS_EX_NOINHERITLAYOUT, CreateWindowEx()
	* fails to create the HWND. The fix is to not use these bits.
	*/
	OSVERSIONINFO os_version;
	os_version.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx((LPOSVERSIONINFO)&os_version);
	// if it is NOT Windows NT
	if ((os_version.dwMajorVersion << 16 | os_version.dwMinorVersion) >= (4 << 16 | 10))  {
		SET_FLAG(style, WS_EX_NOINHERITLAYOUT); 
		if (IS_FLAG_SET(ugl_style, UGL_STYLE_RIGHT_TO_LEFT)) SET_FLAG(style, WS_EX_LAYOUTRTL);
	}
#endif

	if (IS_FLAG_SET(ugl_style, UGL_STYLE_BORDER)) SET_FLAG(style, WS_EX_DLGMODALFRAME); 

#ifndef _WIN32_WCE
	/*
	* Feature in Windows.  When a window that does not have a parent
	* is created, it is automatically added to the Windows Task Bar,
	* even when it has no title.  The fix is to use WS_EX_TOOLWINDOW
	* which does not cause the window to appear in the Task Bar.
	*/
	
	if (!isChild) {
		//if (IS_FLAG_SET(ugl_style, UGL_STYLE_APPLICATION_MODAL)) SET_FLAG(style, WS_EX_TOOLWINDOW);
	}

	/*
	* Bug in Windows 98 and NT.  Creating a window with the
	* WS_EX_TOPMOST extended style can result in a dialog shell
	* being moved behind its parent.  The exact case where this
	* happens is a shell with two dialog shell children where
	* each dialog child has another hidden dialog child with
	* the WS_EX_TOPMOST extended style.  Clicking on either of
	* the visible dialogs causes them to become active but move
	* to the back, behind the parent shell.  The fix is to
	* disallow the WS_EX_TOPMOST extended style on Windows 98
	* and NT.
	*/
	if (isChild) {
		// if Windows 95
		if (os_version.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS) return style;
		if ((os_version.dwMajorVersion << 16 | os_version.dwMinorVersion) < (4 << 16 | 10)) return style;
	}
	//if (parent != null) {
	//	if (OS.IsWin95) return bits;
	//	if ((OS.WIN32_MAJOR << 16 | OS.WIN32_MINOR) < (4 << 16 | 10)) {
	//		return bits;
	//	}
	//}

#endif
#if defined(WIN32_PLATFORM_PSPC) || defined(WIN32_PLATFORM_WFSP)
	//if (IS_FLAG_SET(ugl_style, UGL_STYLE_CLOSE) && isChild) SET_FLAG(style, WS_EX_CAPTIONOKBTN);
	if IS_FLAG_SET(ugl_style, UGL_STYLE_CLOSE) SET_FLAG(style, WS_EX_CAPTIONOKBTN); //fix to bug#120258, replace OK button with 'X'
#endif
	//if (IS_FLAG_SET(ugl_style, UGL_STYLE_APPLICATION_MODAL)) SET_FLAG(style, WS_EX_TOPMOST);
	return style;
}
//========================================================================================

UGL_Int _Shell_Style(UGL_Int ugl_style, BOOL isChild) {
	int style = _Scrollable_Style(ugl_style) & ~(WS_TABSTOP | WS_CHILD);
	
	/* Set the title bits and no-trim bits */
	//if ((style & SWT.NO_TRIM) != 0) return bits;
	//if (ugl_style == 0) return style;
	
	if (ugl_style != 0) {

		/* Set the min and max button bits */
#ifndef _WIN32_WCE	/* Minimize and maximize buttons don't make sense for WinCE devices */
		if (IS_FLAG_SET(ugl_style, UGL_STYLE_MIN)) SET_FLAG(style, WS_MINIMIZEBOX);
		if (IS_FLAG_SET(ugl_style, UGL_STYLE_MAX)) SET_FLAG(style, WS_MAXIMIZEBOX);
#endif 		

		/* On PPC use the WS_EX_CAPTIONOKBTN style instead of the WS_SYSMENU style. */
#if defined(WIN32_PLATFORM_PSPC) || defined(WIN32_PLATFORM_WFSP)
		if (IS_FLAG_SET(ugl_style, UGL_STYLE_CLOSE) && !isChild) SET_FLAG(style, WS_SYSMENU);
#else
		if (IS_FLAG_SET(ugl_style, UGL_STYLE_CLOSE)) SET_FLAG(style, WS_SYSMENU);
#endif

		/* Set the resize, dialog border or border bits */
		//if ((style & SWT.RESIZE) != 0) {
			/*
			* Note on WinCE PPC.  SWT.RESIZE is used to resize
			* the Shell according to the state of the IME.
			* It does not set the WS_THICKFRAME style.
			*/
			//if (!OS.IsPPC) bits |= OS.WS_THICKFRAME;
//#ifndef _WIN32_WCE
//			SET_FLAG(style, WS_THICKFRAME); // makes window resizeable
//#endif
		//} else {
			//if (IS_FLAG_SET(ugl_style, UGL_STYLE_BORDER)) SET_FLAG(style, WS_BORDER);
		//}

	}
	

	/*
	* Feature in WinCE.  Calling CreateWindowEx () with WS_OVERLAPPED
	* and a parent window causes the new window to become a WS_CHILD of
	* the parent instead of a dialog child.  The fix is to use WS_POPUP
	* for a window with a parent.  
	* 
	* Feature in WinCE PPC.  A window without a parent with WS_POPUP
	* always shows on top of the Pocket PC 'Today Screen'. The fix
	* is to not set WS_POPUP for a window without a parent on WinCE
	* devices.
	* 
	* NOTE: WS_POPUP causes CreateWindowEx () to ignore CW_USEDEFAULT
	* and causes the default window location and size to be zero.
	*/
#if defined(_WIN32_WCE) || defined(WIN32_PLATFORM_WFSP)
	//if (OS.IsWinCE) {
		//if (OS.IsSP) return bits | OS.WS_POPUP;

		if (isChild) SET_FLAG(style, WS_POPUP);

		return style;
	//}
#else
	
	/*
	* Use WS_OVERLAPPED for all windows, either dialog or top level
	* so that CreateWindowEx () will respect CW_USEDEFAULT and set
	* the default window location and size.
	* 
	* NOTE:  When a WS_OVERLAPPED window is created, Windows gives
	* the new window WS_CAPTION style bits.  These two constants are
	* as follows:
	* 
	* 	WS_OVERLAPPED = 0
	* 	WS_CAPTION = WS_BORDER | WS_DLGFRAME
	* 
	*/
	SET_FLAG(style, WS_OVERLAPPED);
	SET_FLAG(style, WS_CAPTION);
	//style |= WS_EX_DLGMODALFRAME;
	return style;
#endif
}
//========================================================================================

UGL_Int _Shell_New(UGL_Int parentHandle, UGL_Int displayHandle, UGL_Int ugl_style, UGL_Error uglError) {
	PUGL_Control shell = NULL; 
	PUGL_Display ugl_display = NULL;
	PUGL_Control ugl_parent = NULL;
	HWND hWnd, hWndParent = NULL;
	WNDCLASS lpWndClass;
	
	BOOL is_child = (parentHandle != 0);
	int extended_style = 0;
	int style = 0;
	int bits = 0;
	int flags = 0;
	
	HINSTANCE hInstance;
	
	if (is_child) {
		ugl_parent = UGL_CONTROL(parentHandle);
		hWndParent = ugl_parent->hWnd;
		ugl_display = UGL_DISPLAY(Widget_GetDisplay(parentHandle));
	} else {
		ugl_display = UGL_DISPLAY(displayHandle);
	}

	hInstance = ugl_display->hInstance; 

	// if the current thread is different than the display's thread
	if (GetCurrentThreadId() != (unsigned long)ugl_display->threadID) {
		// block this thread and let the display thread call this function
		UGL_Int args[2] = {parentHandle, (UGL_Int)uglError};
		// initiate creating the shell using our own message id (ie. above WM_USER).
		shell = (PUGL_Control)SendMessage(ugl_display->creation_hWnd, UI_CREATE_SHELL, (WPARAM)ugl_style, (LPARAM)&args);
		return (UGL_Int)shell;
	}
	
	// you can't get down here unless you have come from creationProc in Display
	shell = UGL_CONTROL(calloc(sizeof(UGL_Shell), 1));
	if (shell == NULL) return 0;
	extended_style = _Shell_ExStyle(ugl_style, is_child);
	style = _Shell_Style(ugl_style, is_child);

	hWnd = CreateWindowEx(extended_style, SHELL_WINDOW_CLASS, NULL, style,
      CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, hWndParent, NULL, hInstance, NULL);

   if (!hWnd) {
 		Win32Error_SetError(uglError, GetLastError(), _T("Unable to create window"));
		return NULL_HANDLE;
   }

// post processing?
   	bits = GetWindowLong(hWnd, GWL_STYLE);	
	bits &= ~(WS_OVERLAPPED | WS_CAPTION);

#ifndef _WIN32_WCE
	bits |= WS_POPUP;
#endif

#if defined(WIN32_PLATFORM_PSPC) || defined(WIN32_PLATFORM_WFSP)
	if (is_child && IS_FLAG_SET(ugl_style, UGL_STYLE_TITLE)) bits |= WS_CAPTION;
#else
	if (IS_FLAG_SET(ugl_style, UGL_STYLE_TITLE)) bits |= WS_CAPTION;
#endif

	// Add the border back to the window it was specified because
	// then the WS_CAPTION style was removed avoid the WS_BORDER style gets
	// removed as well
	if ((ugl_style & (UGL_STYLE_BORDER)) == UGL_STYLE_BORDER) bits |= WS_BORDER;

	/*
	* Bug in Windows.  When the WS_CAPTION bits are cleared using
	* SetWindowLong(), Windows does not resize the client area of
	* the window to get rid of the caption until the first resize.
	* The fix is to use SetWindowPos() with SWP_DRAWFRAME to force
	* the frame to be redrawn and resized.
	*/
	SetWindowLong(hWnd, GWL_STYLE, bits);
	flags = SWP_DRAWFRAME | SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE;
	_SetWindowPos(hWnd, 0, 0, 0, 0, 0, flags);

	UGL_WIDGET(shell)->display = ugl_display;
	UGL_WIDGET(shell)->callback_target = NULL;
	UGL_WIDGET(shell)->ugl_style = ugl_style;
	UGL_WIDGET(shell)->widget_dispose = _Dispose_Shell;
	
	shell->parent = ugl_parent;
	shell->hWnd = hWnd;
	shell->widgetProc = NULL;
	shell->redraw_enabled = TRUE;
	shell->background_color = 0;
	shell->foreground_color = 0;
	shell->cursor = NULL;
	shell->inheritsBackground = TRUE;
	shell->sendKeyEvent = _Control_SendKeyEvent;

#ifndef _WIN32_WCE
	UGL_SHELL(shell)->state = NORMAL;
#endif
	
#if (_WIN32_WCE >= 420)
	{
		SHACTIVATEINFO *shActivateInfo = &(UGL_SHELL(shell)->shActivateInfo);
		memset(shActivateInfo, 0, sizeof(*shActivateInfo));	
		shActivateInfo->cbSize = sizeof(*shActivateInfo);
	}
#endif
	
	SetWindowLong(hWnd, GWL_USERDATA, (LONG)shell);

	lpWndClass.lpfnWndProc = 0;
	GetClassInfo(hInstance, SHELL_WINDOW_CLASS, &lpWndClass);
	shell->widgetProc = lpWndClass.lpfnWndProc; 

	// let original display handler create shell first, then set new handler
	SetWindowLong(hWnd, GWL_WNDPROC, (LONG)_Shell_Proc);

#ifdef _WIN32_WCE
#if defined(WIN32_PLATFORM_PSPC)
	// All PocketPC shells should have an empty menu by default
	UGL_SHELL(shell)->emptyMenu = (UGL_Menu*)_MenuBar_New(shell, NULL);
#endif

	if (shell->parent == NULL) {
		// only Toplevel shells on WinCE should always be maximized
		Shell_SetMaximized((UGL_Int)shell, TRUE, uglError);
	}
#endif

	Display_AddShell(ugl_display, shell, uglError);
	return (UGL_Int)shell;

 }

void _Shell_GetInsets(PUGL_Control control, LPRECT rect) {
	_Scrollable_GetInsets(control, rect);

#if _WIN32_WCE && !(WIN32_PLATFORM_PSPC) && !(WIN32_PLATFORM_WFSP) 
	{
		PUGL_Shell shell = UGL_SHELL(control);
		if (shell->ugl_menu != NULL) {
			rect->top += CommandBar_Height(shell->ugl_menu->hwndCB);
		}
	}
#endif 

#ifndef _WIN32_WCE
	// The logic in _Scrollable_GetInsets does not correctly
	// calculate the size of the insets when the shell
	// has a menubar and that menubar is wraped across two 
	// lines.  This code is a fix for that problem (bug 108315)	
	if (GetMenu(control->hWnd) != NULL) {
		RECT windowRect;
		int oldTop, calculatedTopInset;
		GetWindowRect(control->hWnd, &windowRect);
		oldTop = windowRect.top;
		SendMessage(control->hWnd, WM_NCCALCSIZE, 0, (LPARAM)&windowRect);
		calculatedTopInset = windowRect.top - oldTop;
		if (rect->top < (calculatedTopInset)) {
			rect->top = calculatedTopInset;
		}
	}
#endif
}

#ifdef _WIN32_WCE
void _Shell_AdjustChildBounds(PUGL_Control control, LPRECT childBounds) {

#if !(WIN32_PLATFORM_PSPC) && !(WIN32_PLATFORM_WFSP) 
	PUGL_Shell shell = UGL_SHELL(control);
	if (shell->ugl_menu != NULL) {
		int menubarHeight = CommandBar_Height(shell->ugl_menu->hwndCB);
		childBounds->top += menubarHeight;
		childBounds->bottom += menubarHeight;
	}
#endif 	

}
#endif

void _Shell_HandleResize(PUGL_Control shell, ShellState newState) {
	RECT rect;
	int nWidth;
	int nHeight;
#ifndef _WIN32_WCE
	ShellState state = UGL_SHELL(shell)->state;
#endif

	GetWindowRect(shell->hWnd, &rect);
	nWidth = rect.right - rect.left;
	nHeight = rect.bottom - rect.top;

#ifndef _WIN32_WCE
	if (state != newState) {
		if (state == MINIMIZED) {
			ShellCallback(shell->widget.callback_target, UGL_EVENT_SHELL_DEICONIFIED);
		}
		UGL_SHELL(shell)->state = newState;
	}
#endif

	ShellResizeCallback(shell->widget.callback_target, nWidth, nHeight);
}

 //========================================================================================

LRESULT CALLBACK _Shell_Proc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	PUGL_Control ugl_shell = (PUGL_Control)GetWindowLong(hWnd, GWL_USERDATA);
	PUGL_Display ugl_display = UGL_DISPLAY(Widget_GetDisplay((UGL_Int)ugl_shell));
	LRESULT result = 0;
	int i;

	switch (message) {

		case WM_ACTIVATE: {
			if (LOWORD(wParam) == WA_INACTIVE) {
				//SWT saves the focus here. We seem to get away without it.
				ShellCallback(UGL_WIDGET(ugl_shell)->callback_target, UGL_EVENT_SHELL_DEACTIVATED);
			} else {
#if (_WIN32_WCE >= 420)
				SHACTIVATEINFO *shActivateInfo;
#endif

				if (HIWORD(wParam) != 0) return 0; //if the Shell is not minimized...
				ugl_display->activeShell = ugl_shell;
				for(i=0; i<ugl_display->menuItemsLength; i++){
					if(ugl_display->menuItems[i] != NULL){
						if (ugl_display->menuItems[i]->parent_menu->parent == ugl_shell) {
							ugl_display->activeMenuBar = ugl_display->menuItems[i]->parent_menu;
							break;
						}
					}
				}
				ShellCallback(UGL_WIDGET(ugl_shell)->callback_target, UGL_EVENT_SHELL_ACTIVATED);


#if (_WIN32_WCE >= 420)
				{
					/**
					 * On WinCE, SHHandleWMActivate is used to manage the state
					 * of the input window.  
					 * One side effect of SHHandleWMActivate, which is not documented
					 * in MSDN, is that if the window is activated by a mouse click
					 * then the window will be resized to fill the entire client area
					 * of the screen.  This is not the behavior that the Java toolkits
					 * expect so the old window size is saved before and restored after 
					 * the call to SHHandleWMActivate.
					 */
					RECT old;
					GetWindowRect(hWnd, &old);
					shActivateInfo = &(UGL_SHELL(ugl_shell)->shActivateInfo);
					// restore input panel to previous state
					SHHandleWMActivate(hWnd, wParam, lParam, shActivateInfo, 0);
					SetWindowPos(hWnd, NULL, old.left, old.top, (old.right - old.left), (old.bottom - old.top), SWP_NOZORDER | SWP_NOACTIVATE);
				}
#endif
#if defined(WIN32_PLATFORM_PSPC)
				_Shell_EnsureMenuBarIsVisible(UGL_SHELL(ugl_shell), ugl_shell->parent);
#endif
			}
			break;
		}

		case WM_SIZE: {
			switch (wParam) {
				case SIZE_MAXIMIZED: {
					_Shell_HandleResize(ugl_shell, MAXIMIZED);
					break;
				}
				case SIZE_RESTORED: {
					_Shell_HandleResize(ugl_shell, NORMAL);
					break;
				}
				case SIZE_MINIMIZED: {
					// Don't use _Sehll_HandleResize() because a new size
					// does not need to be sent back to Java
					ShellCallback(ugl_shell->widget.callback_target, UGL_EVENT_SHELL_ICONIFIED);
#ifndef _WIN32_WCE
					UGL_SHELL(ugl_shell)->state = MINIMIZED;
#endif
					break;
				}
			}
			break;
		}

		case WM_MOVE: {
			/*
			 * PPro expects us to return the x and y coordinate of the whole window, not 
			 * the client area.
			 */
			RECT rect;
			int xPos;
			int yPos;
			GetWindowRect(hWnd, &rect);
			xPos = rect.left;
			yPos = rect.top;
			ShellMoveCallback(ugl_shell->widget.callback_target, xPos, yPos);
			break;
		}

		case WM_DRAWITEM: {
			LPDRAWITEMSTRUCT draw_struct = (LPDRAWITEMSTRUCT)lParam;
			if (draw_struct->CtlType == ODT_MENU) {
				PUGL_MenuItem ugl_menuItem = Display_GetMenuItem(ugl_shell->widget.display, draw_struct->itemData);
				if ((ugl_menuItem != NULL) && (ugl_menuItem->wmDrawChild != NULL)) 
					result = ugl_menuItem->wmDrawChild((UGL_Int)ugl_menuItem, wParam, lParam);
			}
			break;
		} 

		case WM_MEASUREITEM: {
			LPMEASUREITEMSTRUCT measure_struct = (LPMEASUREITEMSTRUCT)lParam;
			if (measure_struct->CtlType == ODT_MENU) {
				PUGL_MenuItem ugl_menuItem = Display_GetMenuItem(ugl_shell->widget.display, measure_struct->itemData);
				if ((ugl_menuItem != NULL) && (ugl_menuItem->wmMeasureChild != NULL)) {
					result = ugl_menuItem->wmMeasureChild((UGL_Int)ugl_menuItem, wParam, lParam);
					return TRUE;
				}
			} 
			break;
		} 

#if (_WIN32_WCE >= 420)
		case WM_SETTINGCHANGE:
			if (wParam == SPI_SETSIPINFO) {
				RECT old;
				PUGL_Shell shell = UGL_SHELL(ugl_shell);
				SHACTIVATEINFO *shActivateInfo = &(shell->shActivateInfo);

				// resize shell and store state when input panel is shown/hidden
				// The shell should not be resized if it is not fullscreen.  This covers
				// both child windows and if we ever support non-fullscreen top-level windows
				if (_Shell_GetMaximized(shell) == FALSE) GetWindowRect(hWnd, &old);
				SHHandleWMSettingChange(hWnd, wParam, lParam, shActivateInfo);
				if (_Shell_GetMaximized(shell) == FALSE)  SetWindowPos(hWnd, NULL, old.left, old.top, (old.right - old.left), (old.bottom - old.top), SWP_NOZORDER | SWP_NOACTIVATE);
#if defined(WIN32_PLATFORM_PSPC)
                if( GetForegroundWindow() == hWnd) _Shell_EnsureMenuBarIsVisible(shell, ugl_shell->parent);
#endif
				return 0;
			}
			break;
#endif
#if defined(_WIN32_WCE)
		case WM_COMMAND: {
			PUGL_Shell shell = UGL_SHELL(ugl_shell);
#if defined(WIN32_PLATFORM_PSPC) || defined(WIN32_PLATFORM_WFSP)
			/*
			* Note in WinCE PPC:  Close the Shell when the "Done Button" has
			* been pressed. lParam is either 0 (PocketPC 2002) or the handle
			* to the Shell (PocketPC).
			*/
			int loWord = wParam & 0xFFFF;
			if (loWord == IDOK && (lParam == 0 || lParam == hWnd)) {
				PostMessage (hWnd, WM_CLOSE, 0, 0);
				return 0;			
			}
#endif

			if (shell->ugl_menu != NULL) {
				// For menu items that are placed directly on the
				// menubar, the window that is supplied in the
				// lParam is NOT the menu hwnd as would be expected, but
				// actually the one of the child widgets of the menubar.
				HWND menubarHWND = shell->ugl_menu->hwndCB;
				if ((menubarHWND == (HWND)lParam) || (IsChild(menubarHWND, (HWND)lParam))) {
					WORD menuId = wParam & 0xFFFF;
					_Control_ProcessMenuItemCommand(ugl_display, menuId, wParam, lParam);
				}
				
			}
			break;
		}
#endif

		case UI_MOVE_ABOVE:{
			PUGL_Control sibling = (PUGL_Control)wParam;
			_Shell_MoveAbove(ugl_shell, sibling, (UGL_Error)lParam); 
			return 0;
		} break; 

		case UI_SCROLLABLE_GETINSETS: {
			_Shell_GetInsets(ugl_shell, (LPRECT)wParam);			
			return 0;
		}

		case UI_SET_ENABLED: {
			BOOL enable = (BOOL)wParam;
			PUGL_Display display = Control_GetDisplay(ugl_shell);
			if (!Display_EnableShell(display, ugl_shell, enable)) {
				// the display does not want the enabled status of this
				// control to change so it is important that no further processing
				// happen on this message
				return 0;
			}
			break;
		}

		case UI_SET_VISIBLE: {
			// It is important that the enabled status of the shells be updated 
			// before this shell becomes visible/invisible because if no other
			// shell is enabled when the modal dialog is made invisble then no
			// windows are brought to the front.
			BOOL visible = (BOOL)wParam;
			UGL_Error* error = (UGL_Error*)lParam;
			if (IS_FLAG_SET(ugl_shell->widget.ugl_style, UGL_STYLE_APPLICATION_MODAL)) {
				PUGL_Display display = Control_GetDisplay(ugl_shell);
				Display_UpdateModalShellVisibility(display, ugl_shell, visible, *error);
			}

#ifndef _WIN32_WCE
			if (visible) {
				if ((UGL_SHELL(ugl_shell)->state != MINIMIZED) && (IsIconic(hWnd))) {
					ShellCallback(ugl_shell->widget.callback_target, UGL_EVENT_SHELL_DEICONIFIED);
				}

				if (UGL_SHELL(ugl_shell)->state != NORMAL) {
					if (UGL_SHELL(ugl_shell)->state == MAXIMIZED) {
						ShowWindow(hWnd, SW_MAXIMIZE);
					} else {
						ShowWindow(hWnd, SW_SHOWMINNOACTIVE);
					}
					return 0;
				} 
			}
#endif

			break;
		}

#ifdef _WIN32_WCE
		case UI_SET_BOUNDS: {
			LPRECT bounds = (LPRECT)wParam;
			UGL_Error error = (UGL_Error)lParam;
			int x = bounds->left;
			int y = bounds->top;
			int width = bounds->right - bounds->left;
			int height = bounds->bottom - bounds->top;

			if (ugl_shell->parent != NULL) {
				if (!SetWindowPos(hWnd, NULL, x, y, width, height, SWP_NOZORDER)) {
					Win32Error_SetError(error, GetLastError(), _T("Could not change the size of the shell"));
				}
			} else {
				// Do nothing since top-level windows are always fullscreen on WinCE
				// Even though the size is not changing the real size needs to
				// be reported back to Java.  If this does not happen Java will think
				// that the frame has taken the new size and layouts will not
				// work correctly.
			}

			Control_GetBounds((UGL_Int) ugl_shell, &x, &y, &width, &height, error);
//			ShellResizeCallback(ugl_shell->widget.callback_target, width, height);
//			ShellMoveCallback(ugl_shell->widget.callback_target, x, y);
			return 0;
		}
#endif
		case UI_GET_MINIMUM_SIZE: {
			POINT* point = (POINT*)wParam;
			_Shell_GetMinimumSize((UGL_Int)ugl_shell, point, (UGL_Error)lParam);
			return 0;
		}

#ifdef _WIN32_WCE
		case UI_ADJUST_CHILD_BOUNDS: {
			// Child controls need to have their bounds adjusted before they are set
			// depending on whether the shell has a menu or not.  The menubar is considered 
			// part of the client area but from a Java perspective it should not be
			_Shell_AdjustChildBounds(ugl_shell, (LPRECT)wParam);
			return 0;
		}
#endif
	}


	// the ugl_display->windowProc will call the DefWindowProc if the message
	// cannot be handled so it is possible to return the value without checking
	// it's value
	return _Composite_Proc(hWnd, message, wParam, lParam);
}
//========================================================================================

void _Dispose_Shell(UGL_Int handle, UGL_Error uglError) {
	PUGL_Shell shell = UGL_SHELL(handle);
	PUGL_Control ugl_control = UGL_CONTROL(handle);
	Display_RemoveShell(Control_GetDisplay(ugl_control), ugl_control);

	if (shell->accel_entries != NULL) {
		free(shell->accel_entries);
	}

	if (shell->accelerator_table != 0) {
		DestroyAcceleratorTable(shell->accelerator_table);
	}

#if defined(WIN32_PLATFORM_PSPC)
	_Dispose_Menu((UGL_Int)shell->emptyMenu, uglError);
#endif

	_Dispose_Scrollable(handle, uglError);
}
//========================================================================================


//========================================================================================

UGL_Int Shell_NewChild(UGL_Int parent, UGL_Int ugl_style, UGL_Error uglError) {
	return _Shell_New(parent, 0, ugl_style, uglError);
}
////========================================================================================
UGL_Boolean Shell_GetMinimized(UGL_Int handle, UGL_Error uglError) {
	PUGL_Control shell = (PUGL_Control)handle;
	HWND hWnd = shell->hWnd;


#ifdef _WIN32_WCE
	return UGL_SHELL(handle)->state == MINIMIZED;
#else
	if (IsWindowVisible (hWnd)) {
		return IsIconic(hWnd);
	} else {
		return UGL_SHELL(handle)->state == MINIMIZED;
	}
#endif
	//return swFlags == OS.SW_SHOWMINNOACTIVE;
}
//========================================================================================

UGL_Boolean _Shell_GetMaximized(PUGL_Shell shell) {
	HWND hWnd = shell->scrollable.control.hWnd;


#ifdef _WIN32_WCE
	return shell->state == MAXIMIZED;
#else
	if (IsWindowVisible (hWnd)) {
		return IsZoomed(hWnd);
	} else {
		return shell->state == MAXIMIZED;
	}
#endif

}
void _Shell_GetMinimumSize(UGL_Int handle, POINT *point, UGL_Error uglError) {
	PUGL_Shell shell = (PUGL_Shell)handle;
	PUGL_Widget shellWidget = (PUGL_Widget)handle;
	PUGL_Control shellControl = (PUGL_Control)handle;
	int ugl_style = shellWidget->ugl_style;
		 
#ifndef _WIN32_WCE
	int widthLimit, heightLimit;
	BOOL trim;

	trim = (IS_FLAG_SET(ugl_style, UGL_STYLE_TITLE)) || 
		(IS_FLAG_SET(ugl_style, UGL_STYLE_CLOSE)) ||
		(IS_FLAG_SET(ugl_style, UGL_STYLE_MIN)) ||
		(IS_FLAG_SET(ugl_style, UGL_STYLE_MAX));
	if (!IS_FLAG_SET(ugl_style, UGL_STYLE_NO_TRIM) && trim) {
		int bits1 = GetWindowLong (shellControl->hWnd, GWL_STYLE);
		widthLimit = GetSystemMetrics (SM_CXMINTRACK);
		if ((bits1 & WS_THICKFRAME) != 0) {
			heightLimit = GetSystemMetrics (SM_CYMINTRACK);
		} else {
			RECT rect;
			int bits2 = GetWindowLong (shellControl->hWnd, GWL_EXSTYLE);
			AdjustWindowRectEx ((LPRECT)&rect, bits1, FALSE, bits2);
			heightLimit = rect.bottom - rect.top;
		}
		point->x = widthLimit;
		point->y = heightLimit;
	} else {
		point->x = 0;
		point->y = 0;
	}
#else
		point->x = 0;
		point->y = 0;
#endif
}
//========================================================================================
void Shell_SetMinimized(UGL_Int handle, UGL_Boolean minimized, UGL_Error uglError) {
	PUGL_Control shell = (PUGL_Control)handle;
	HWND hWnd = shell->hWnd;


#ifndef _WIN32_WCE

	int swFlags = minimized ? SW_SHOWMINNOACTIVE : SW_RESTORE;

	UGL_SHELL(handle)->state = minimized ? MINIMIZED : NORMAL;
	
	if (!IsWindowVisible(hWnd)) return;
	if (minimized == IsIconic(hWnd)) return;

	// The following line has been slightly changed from SWT.  In SWT
	// GetActiveWindow is used but GetActiveWindow only works if the
	// current thread is the thread that owns the active window.  Changing
	// the call to GetForegroundWindow keeps the same logic, but works from any
	// thread.
	if (swFlags == SW_SHOWMINNOACTIVE && hWnd == GetForegroundWindow()) {
		swFlags = SW_MINIMIZE;
	}

	if (!minimized) {
		// Since the state has already been updated to reflect the minimized state
		// the event from the OS will not generate a deiconified event.  A deiconified
		// event is still desired so we'll force one here
		ShellCallback(shell->widget.callback_target, UGL_EVENT_SHELL_DEICONIFIED);
	}

	ShowWindow(hWnd, swFlags);
	UpdateWindow(hWnd);
#else
		PUGL_Shell shellmin = UGL_SHELL(handle);
		if (minimized) {		
			shellmin->state = MINIMIZED;			
			ShowWindow(hWnd,SW_MINIMIZE);
		} else {
			if(shellmin->state == MINIMIZED){
				shellmin->state = NORMAL;
				ShowWindow(hWnd, SW_RESTORE);	
			}
		}
#endif
	
}
//========================================================================================

void Shell_SetImage(UGL_Int shellHandle, UGL_Int imageHandle, UGL_Error uglError) {
	/*
	* Feature in WinCE.  WM_SETICON and WM_GETICON set the icon
	* for the window class, not the window instance.  This means
	* that it is possible to set an icon into a window and then
	* later free the icon, thus freeing the icon for every window.
	* The fix is to avoid the API.
	* 
	* On WinCE PPC, icons in windows are not displayed.
	*/
#ifdef _WIN32_WCE
	return;
#else

	PUGL_Control ugl_shell = UGL_CONTROL(shellHandle);
	PUGL_Image ugl_image = UGL_IMAGE(imageHandle);

	ICONINFO icon_info;
	HICON hIcon;

	if (imageHandle == 0) {
		// remove icon from shell
		hIcon = NULL;
	} else {
		memset(&icon_info, 0, sizeof(ICONINFO));
		icon_info.fIcon = TRUE;
		icon_info.hbmColor = ugl_image->image_handle;
		icon_info.hbmMask = ugl_image->image_handle;

		hIcon = CreateIconIndirect(&icon_info);
		if (hIcon == NULL) {
			Win32Error_SetError(uglError, GetLastError(), _T("Unable to create icon from image!"));
			return;
		}
	}

	SendMessage(ugl_shell->hWnd, WM_SETICON, ICON_SMALL, (LPARAM)hIcon);
	SendMessage(ugl_shell->hWnd, WM_SETICON, ICON_BIG, (LPARAM)hIcon);

	/*
	* Bug in Windows.  When WM_SETICON is used to remove an
	* icon from the window trimmings for a window with the
	* extended style bits WS_EX_DLGMODALFRAME, the window
	* trimmings do not redraw to hide the previous icon.
	* The fix is to force a redraw.
	*/

	RedrawWindow(ugl_shell->hWnd, NULL, 0, RDW_FRAME | RDW_INVALIDATE);

#endif // defined _WIN32_WCE
}
//========================================================================================

void Shell_SetResizable(UGL_Int shellHandle, UGL_Boolean resizeable, UGL_Error uglError)  {
	PUGL_Shell ugl_shell = UGL_SHELL(shellHandle);
	int style_bits = 0;
	HWND hwnd = UGL_CONTROL(ugl_shell)->hWnd;


#ifdef _WIN32_WCE
	if (GetParent(hwnd) != NULL) {
#endif
	style_bits = GetWindowLong(hwnd, GWL_STYLE);
	
	if (resizeable) SET_FLAG(style_bits, WS_THICKFRAME);
	else UNSET_FLAG(style_bits, WS_THICKFRAME);

	SetWindowLong(hwnd, GWL_STYLE, style_bits);
	SetWindowPos(hwnd, NULL, 0,0,0,0, SWP_NOMOVE | SWP_NOOWNERZORDER | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED | SWP_NOACTIVATE);
#ifdef _WIN32_WCE
	}
#endif
}
//========================================================================================


void _Shell_MoveAbove(PUGL_Control control, PUGL_Control sibling, UGL_Error error) {
	_Control_MoveAbove(control, sibling, error);
	if (sibling == NULL) {
		SetForegroundWindow(control->hWnd);
	}
}


//========================================================================================

void Shell_SetMenuBar(UGL_Int shellHandle, UGL_Int menuBarHandle, UGL_Error uglError)  {
	PUGL_Shell ugl_shell = UGL_SHELL(shellHandle);
	PUGL_Menu ugl_menu = UGL_MENU(menuBarHandle);
	PUGL_Control ugl_control = UGL_CONTROL(shellHandle);

#if defined(WIN32_PLATFORM_PSPC)
	/*
	* Note in WinCE PPC.  The menu bar is a separate popup window.
	* If the shell is full screen, resize its window to leave
	* space for the menu bar.
	*/
	PUGL_Menu old_menu = ugl_shell->ugl_menu != NULL ? ugl_shell->ugl_menu : ugl_shell->emptyMenu;
	if (old_menu != NULL) ShowWindow(old_menu->hwndCB, SW_HIDE);
	
	ugl_shell->ugl_menu = ugl_menu;

	_Shell_EnsureMenuBarIsVisible(ugl_shell, ugl_control->parent);
#endif // if PPC

#ifdef WIN32_PLATFORM_WFSP

	PUGL_Menu old_menu = ugl_shell->ugl_menu;
	if (old_menu != NULL) ShowWindow(old_menu->hwndCB, SW_HIDE);
	ugl_shell->ugl_menu = ugl_menu;
	if (ugl_menu != NULL) ShowWindow(ugl_menu->hwndCB, SW_SHOW);
		
#endif // if SP

#if _WIN32_WCE && !(WIN32_PLATFORM_PSPC) && !(WIN32_PLATFORM_WFSP) 
	HWND shellHwnd = ugl_shell->scrollable.control.hWnd;
	PUGL_Menu old_menu = ugl_shell->ugl_menu;
	if (old_menu != NULL) {
		CommandBar_Show(old_menu->hwndCB, FALSE);

		// Shift the child widgets up to take into consideration the height of the commandbar which has been
		// removed
		ScrollWindowEx(shellHwnd, 0, -CommandBar_Height(old_menu->hwndCB), NULL, NULL, 0, NULL, SW_SCROLLCHILDREN);
	}
	ugl_shell->ugl_menu = ugl_menu;
	if (ugl_menu != NULL) {
		// Shift the child widgets down to take make room for the new CommandBar.  For some reason
		// the CommandBar will return the wrong location if it is not at (0,0).  The CommandBar may not
		// be at (0,0) if a second menubar has been created on the frame and is being made visible.  Moving
		// the CommandBar back to (0,0) before getting the height will fix this problem.
		SetWindowPos(ugl_menu->hwndCB, HWND_TOPMOST, 0,0,0,0, SWP_NOSIZE);
		ScrollWindowEx(shellHwnd, 0, CommandBar_Height(ugl_menu->hwndCB), NULL, NULL, 0, NULL, SW_SCROLLCHILDREN);
	
		// Bring the CommandBar to the front and make sure that it is at the top of the window.  This second
		// call is needed because the ScrollWindowEx call above will move the CommandBar in addition to the other
		// widgets.
		SetWindowPos(ugl_menu->hwndCB, HWND_TOPMOST, 0,0,0,0, SWP_NOSIZE);

		CommandBar_Show(ugl_menu->hwndCB, TRUE);
	}

	// On WinCE the application should only have a title bar, or a menu bar.  Both at the same
	// time wastes space and should be avoided.  This is only true for maximized though shells since
	// shells that don't fill the entire screen look weird without a titlebar
	if (_Shell_GetMaximized(ugl_shell) == TRUE) {
		LONG styleBits = GetWindowLong(shellHwnd, GWL_STYLE);
		
		if (menuBarHandle == 0) {
			styleBits |= WS_CAPTION;
		} else {
			styleBits &= ~WS_CAPTION;

			CommandBar_AddAdornments(ugl_menu->hwndCB, 0,0);
		}

		SetWindowLong(shellHwnd, GWL_STYLE, styleBits);

		// Force the update of the frame
		_SetWindowPos(shellHwnd, 0, 0, 0, 0, 0, SWP_DRAWFRAME | SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE);
	}
#endif

#ifndef _WIN32_WCE
	SetMenu(ugl_shell->scrollable.control.hWnd, (ugl_menu != NULL ? ugl_menu->hMenu : 0));
#endif // if WINCE
}
//========================================================================================

#define ACCEL_TABLE_INIT_SIZE 10
#define ACCEL_TABLE_INCREMENT 5

void _Shell_SetAccelerator(PUGL_Shell shell, LPACCEL accel, UGL_Error error) {
	// Try to update the entry in place, or signal that a new
	// entry needs to be created to hold this accel
	BOOL needsAdd = TRUE;
	if (shell->accel_entries != 0) {
		int i;
		for (i=0;i<shell->accel_entries_used;i++) {
			if (shell->accel_entries[i].cmd == accel->cmd) {
				shell->accel_entries[i].fVirt = accel->fVirt;
				shell->accel_entries[i].key = accel->key;
				needsAdd = FALSE;
				break;
			}
		}
	}

	// If the accelerator does not have any entry then a new one must
	// be created
	if (needsAdd) {
		int i;

		// Either a new table needs to be created or we just need
		// to make sure that there is enough space at the end of the table
		if (shell->accel_entries == NULL) {
			shell->accel_entries = (LPACCEL)calloc(sizeof(ACCEL), ACCEL_TABLE_INIT_SIZE);
			if (shell->accel_entries == NULL) {
				Win32Error_SetError(error, UGL_ERROR_OUT_OF_MEMORY, _T("Not enough space to create accelerator table"));
				return;
			}

			shell->accel_entries_space = ACCEL_TABLE_INIT_SIZE;
			shell->accel_entries_used = 0;
		} else {
			if (shell->accel_entries_used == shell->accel_entries_space) {
				// The table is maxed out and is must be resized to fit the new entry
				int newSize = shell->accel_entries_space + ACCEL_TABLE_INCREMENT;
				LPACCEL newTable = (LPACCEL)calloc(sizeof(ACCEL), newSize);
				LPACCEL oldTable = NULL;
				if (newTable == NULL) {
					Win32Error_SetError(error, UGL_ERROR_OUT_OF_MEMORY, _T("Not enough space to grow accelerator table"));
					return;
				}
				memcpy(newTable, shell->accel_entries, shell->accel_entries_used * sizeof(ACCEL));
				oldTable = shell->accel_entries;
				shell->accel_entries = newTable;
				shell->accel_entries_space = newSize;

				// The old table is not needed anymore
				free(oldTable);				
			}
		}

		// By now we've either created a new table, or ensured there is enough space
		// at the end of the current table to add a entry
		i = shell->accel_entries_used;
		shell->accel_entries[i].cmd = accel->cmd;
		shell->accel_entries[i].fVirt = accel->fVirt;
		shell->accel_entries[i].key = accel->key;
		shell->accel_entries_used++;
	}

	// Destory the cached accelerator table because of the change
	if (shell->accelerator_table != 0) DestroyAcceleratorTable(shell->accelerator_table);
	shell->accelerator_table = 0;
}

void _Shell_RemoveAccelerator(PUGL_Shell shell, WORD cmd) {
	int i;
	for (i=0;i<shell->accel_entries_used;i++) {
		if (shell->accel_entries[i].cmd == cmd) {
			// The command was in the list, so all entries
			// after the comment should be shifted down one
			// spot and the size shrunk by 1
			int nextEntry = i+1;
			memmove(shell->accel_entries + i, shell->accel_entries + nextEntry, (shell->accel_entries_used - nextEntry) * sizeof(ACCEL));
			shell->accel_entries_used--;

			// Since the list has been modifed, the cached table
			// should be removed
			if (shell->accelerator_table != 0) {
				DestroyAcceleratorTable(shell->accelerator_table);
			}
		}
	}
}

HACCEL _Shell_GetAcceleratorTable(PUGL_Shell shell) {
	if (shell->accelerator_table == 0) {
		if (shell->accel_entries_used == 0) {
			return 0;
		} else {
			shell->accelerator_table = CreateAcceleratorTable(shell->accel_entries, shell->accel_entries_used);
		}
	}
	return shell->accelerator_table;
}




//========================================================================================

UGL_Boolean Shell_GetMaximized(UGL_Int handle, UGL_Error uglError) {
	PUGL_Shell shell = (PUGL_Shell)handle;
	return _Shell_GetMaximized(shell);
}

//========================================================================================

/**
 * Makes a button the default for this shell.  Default buttons may
 * have a special appearance, or they may take the initial focus
 * from the shell.
 * 
 * @param buttonHandle the button to be made default, or 0 if no default
 *                     button should be used.  This handle will only
 *                     be buttons created with PushButton_New()
 */
void Shell_SetDefaultButton(UGL_Int shellhandle, UGL_Int buttonHandle, UGL_Error uglError) {
	PUGL_Control shell = UGL_CONTROL(shellhandle);
	
	// Unset the old button
	HWND oldButton = (HWND)SendMessage(shell->hWnd, DM_GETDEFID, 0, 0);
	if (oldButton != NULL) {
		int oldbits = GetWindowLong(oldButton, GWL_STYLE);
		oldbits &= ~BS_DEFPUSHBUTTON;
		SendMessage(oldButton, BM_SETSTYLE, oldbits, 1);
	}

	// Update the new button
	if (buttonHandle != 0) {
		PUGL_Control button = UGL_CONTROL(buttonHandle);
		int bits = GetWindowLong(button->hWnd, GWL_STYLE);
		bits |= BS_DEFPUSHBUTTON;
		SendMessage(button->hWnd, BM_SETSTYLE, bits, 1);

		SendMessage(shell->hWnd, DM_SETDEFID, (WPARAM)button->hWnd, 0);
	} else {
		SendMessage(shell->hWnd, DM_SETDEFID, 0, 0);
	}
}

